from __future__ import unicode_literals

from django.db import models

from guardian.shortcuts import get_objects_for_user

from .tasks import (begin_election_proposition, end_election_proposition)

from evotecore.utils import compare_data


class ElectionQuerySet(models.QuerySet):

    def closed(self):
        """Return elections on state CLOSED"""
        return self.filter(state=self.model.State.CLOSED)

    def in_to_vote(self):
        """Return elections on state IN_TO_VOTE"""
        return self.filter(state=self.model.State.IN_TO_VOTE)

    def eligible(self, user):
        """Return elections eligible for the user"""
        return get_objects_for_user(
            user, "vote_election", self.all(), use_groups=False,
            with_superuser=False)


class ElectoralRollQuerySet(models.QuerySet):

    def get_published_of_process(self):
        """Get published electoral roll on the process"""
        return self.filter(published_on__isnull=False).first()

    def get_published(self, election_id):
        """Get published electoral roll on the election"""
        return self.filter(
            election_id=election_id, published_on__isnull=False).first()

    def get_to_publish(self, election_id):
        """Get electoral roll that can be published"""
        to_publish = self.filter(
            election_id=election_id, published_on__isnull=True, imported=True)
        published = self.get_published(election_id)
        if published:
            to_publish = to_publish.filter(created__gt=published.created)
        return to_publish.first()

    def is_importing(self, election_id):
        """ """
        return self.filter(election_id=election_id, imported=None)


class CandidateQuerySet(models.QuerySet):

    def write_in(self):
        """ """
        return self.filter(write_in=True).order_by("designation")

    def not_write_in(self):
        """ """
        return self.filter(write_in=False)


class CandidateBaseManager(models.Manager):
    """ """

    def bulk_import(self, election, data):
        """ """
        candidates = []
        candidates_to_create = []
        for row in data:
            try:
                candidate = self.get(election=election,
                                     personnel_number=row['personnel_number'])
                self.efficient_update(candidate, row, [
                    "designation"
                ])
                candidates.append(candidate)
            except self.model.DoesNotExist:
                candidates_to_create.append(self.model(
                    write_in=True,
                    election=election,
                    designation=row["designation"],
                    personnel_number=row['personnel_number']
                ))
        candidates += self.bulk_create(candidates_to_create)
        # before write in imported candidates cleared
        candidate_ids = [c.pk for c in candidates]
        self.write_in().filter(election=election)\
            .exclude(pk__in=candidate_ids).delete()
        return candidates

    def efficient_update(self, voter, data, keys):
        """Only update if data changed, otherwise do nothing"""
        need_update = not compare_data(voter, data, keys)
        if need_update:
            for key in keys:
                setattr(voter, key, data[key])
            voter.save(update_fields=keys)


CandidateManager = CandidateBaseManager.from_queryset(CandidateQuerySet)


class ElectionTaskManager(object):

    def begin_election_proposition_task(self, election_id, eta):
        begin_election_proposition.si(election_id).apply_async(eta=eta)

    def end_election_proposition_task(self, election_id, eta):
        end_election_proposition.si(election_id).apply_async(eta=eta)


election_task_manager = ElectionTaskManager()
