from __future__ import unicode_literals

from django.apps import apps
from django.conf import settings
from celery.schedules import crontab
from django.utils import timezone
from django.utils.translation import ugettext as _

from evotevotings.elections.importers import ElectoralRollImporter
from celery.utils.log import get_task_logger
from evote.celery import app
from django_fsm import can_proceed

from .mailings import (ElectionNotificationDateMail)


logger = get_task_logger(__name__)


def get_election(pk):
    Election = apps.get_model("evoteelections.Election")  # noqa
    return Election.objects.get(pk=pk)


def get_electoral_roll(pk):
    ElectoralRoll = apps.get_model("evoteelections.ElectoralRoll")  # noqa
    return ElectoralRoll.objects.get(pk=pk)


@app.task(ignore_result=True)
def begin_election_proposition(election_id):
    election = get_election(election_id)
    if can_proceed(election.begin_proposition):
        election.begin_proposition()
        election.save()


@app.task(ignore_result=True)
def end_election_proposition(election_id):
    election = get_election(election_id)
    if can_proceed(election.end_proposition):
        election.end_proposition()
        election.save()


@app.task(ignore_result=True)
def open_election_voting(election_id):
    election = get_election(election_id)
    if can_proceed(election.voting):
        election.voting()
        election.save()


@app.task(ignore_result=True)
def close_election_voting(election_id):
    election = get_election(election_id)
    if can_proceed(election.counting):
        election.counting()
        election.save()


@app.task(ignore_result=True)
def close_elections(election_id):
    election = get_election(election_id)
    if can_proceed(election.close):
        election.close()
        election.save()


@app.task(ignore_result=True)
def import_electoral_roll(electoral_roll_id):
    electoral_roll = get_electoral_roll(electoral_roll_id)
    try:
        importer = ElectoralRollImporter(electoral_roll.election)
        importer.run(electoral_roll.document.document)
    except Exception:
        logger.exception("Error importing electoral roll")
    finally:
        electoral_roll.imported = True
        electoral_roll.save()


@app.on_after_finalize.connect
def setup_periodic_tasks(sender, **kwargs):
    notify_hour = getattr(settings, "EVOTE_ELECTION_NOTIFICATION_HOUR", 7)
    notify_minutes = getattr(
        settings, "EVOTE_ELECTION_NOTIFICATION_MINUTES", 0)
    sender.add_periodic_task(
        crontab(hour=notify_hour, minute=notify_minutes),
        send_elections_notifications.s(),
    )


@app.task
def send_elections_notifications():
    sent = 0
    Election = apps.get_model("evoteelections.Election") # NOQA
    qs = Election.objects.filter(state__gte=Election.State.OPEN)
    fields = {
        'proposition_begin_date': {
            'subject': _('Evote: Proposition begin notification'),
            'text': _('begin the proposition period')
        },
        'proposition_end_date': {
            'subject': _('Evote: Proposition end notification'),
            'text': _('end the proposition period')
        },
        'minute_end_date': {
            'subject': _('Evote: Minute end notification'),
            'text': _('end the minute period')
        },
        'voting_begin_date': {
            'subject': _('Evote: Voting begin notification'),
            'text': _('begin the voting period')
        },
        'roll_begin_date': {
            'subject': _('Evote: Roll begin notification'),
            'text': _('begin the roll period')
        },
        'roll_end_date': {
            'subject': _('Evote: Roll end notification'),
            'text': _('end the roll period')
        }
    }
    days = getattr(settings, "EVOTE_ELECTION_NOTIFICATION_DAYS_BEFORE", [3, 1])
    for election in qs:
        for key in fields:
            date = getattr(election, key)
            for day in days:
                notify_date = date - timezone.timedelta(days=day)
                if notify_date.date() == timezone.now().date():
                    ElectionNotificationDateMail(
                        subject=fields[key]['subject'],
                        context={
                            'object': election,
                            'days': day,
                            'text': fields[key]['text']
                        })\
                        .send(election.electoral_process)
                    sent += 1
    return sent
